@@ -3,8 +3,10 @@ import matplotlib.pyplot as plt
 import pandas as pd
 from matplotlib import cm
 
-def Trajectory (data,bodyPartTraj,bodyPartBox,cmapType='viridis',figureTitle=None, 
-                hSize=6,wSize=8,fontsize=15,invertY=True,saveName=None,figformat=".eps"):
+def Trajectory(data,bodyPartTraj,bodyPartBox, **kwargs):
+  # ,start=None,end=None,fps=30,cmapType='viridis',
+                # figureTitle=None,hSize=6,wSize=8,fontsize=15,invertY=True,saveName=None,
+                # figformat=".eps"):
     """
     Plots the trajectory of the determined body part.
 
@@ -17,6 +19,14 @@ def Trajectory (data,bodyPartTraj,bodyPartBox,cmapType='viridis',figureTitle=Non
     bodyPartBox : str
         The body part you want to use to estimate the limits of the environment, 
         usually the base of the tail is the most suitable for this determination.
+    start : int, optional
+        Moment of the video you want tracking to start, in seconds. If the variable 
+        is empty (None), the entire video will be processed.
+    end : int, optional
+        Moment of the video you want tracking to end, in seconds. If the variable is 
+        empty (None), the entire video will be processed.
+    fps : int
+        The recording frames per second.
     cmapType : str, optional
         matplotlib colormap.
     figureTitle : str, optional
@@ -53,11 +63,49 @@ def Trajectory (data,bodyPartTraj,bodyPartBox,cmapType='viridis',figureTitle=Non
     import matplotlib.pyplot as plt
     import pandas as pd
     from matplotlib import cm
+    from mpl_toolkits.axes_grid1 import make_axes_locatable
+
+    saveName= kwargs.get('saveName')
+    start= kwargs.get('start')
+    end= kwargs.get('end')
+    figureTitle = kwargs.get('figureTitle')
+    fps = kwargs.get('fps')
+    ax = kwargs.get('ax')
+    if type(fps) == type(None):
+      fps = 30
+    cmapType = kwargs.get('cmapType')
+    if type(cmapType) == type(None):
+      cmapType = 'viridis'
+    hSize = kwargs.get('hSize')
+    if type(hSize) == type(None):
+      hSize = 6
+    wSize = kwargs.get('wSize')
+    if type(wSize) == type(None):
+      wSize = 8
+    bins = kwargs.get('bins')
+    if type(bins) == type(None):
+      bins = 30
+    fontsize = kwargs.get('fontsize')
+    if type(fontsize) == type(None):
+      fontsize = 15
+    invertY = kwargs.get('invertY')
+    if type(invertY) == type(None):
+      invertY = True
+    figformat = kwargs.get('figformat')
+    if type(figformat) == type(None):
+      figformat = '.eps'
+
     values = (data.iloc[2:,1:].values).astype(np.float)
     lista1 = (data.iloc[0][1:].values +" - " + data.iloc[1][1:].values).tolist()
 
-    x = values[:,lista1.index(bodyPartTraj+" - x")]
-    y = values[:,lista1.index(bodyPartTraj+" - y")]
+    if type(start) == type(None):
+        x = values[:,lista1.index(bodyPartTraj+" - x")]
+        y = values[:,lista1.index(bodyPartTraj+" - y")]
+    else:
+        init = int(start*fps)
+        finish = int(end*fps)
+        x = values[:,lista1.index(bodyPartTraj+" - x")][init:finish]
+        y = values[:,lista1.index(bodyPartTraj+" - y")][init:finish]
 
     cmap = plt.get_cmap(cmapType)
 
@@ -67,35 +115,42 @@ def Trajectory (data,bodyPartTraj,bodyPartBox,cmapType='viridis',figureTitle=Non
     baixo = values[:,lista1.index(bodyPartBox+" - y")].min()
     cima = values[:,lista1.index(bodyPartBox+" - y")].max()
 
-    plt.rcParams["font.family"] = "Arial"
-    plt.figure(figsize=(wSize, hSize), dpi=80)
-    plt.title(figureTitle, fontsize=fontsize)
-    plt.scatter(x, y, c=c, cmap=cmap, s=3)
-    plt.plot([esquerda,esquerda] , [baixo,cima],"r")
-    plt.plot([esquerda,direita]  , [cima,cima],"r")
-    plt.plot([direita,direita]   , [cima,baixo],"r")
-    plt.plot([direita,esquerda]  , [baixo,baixo],"r")
-    #ax1.set_ylim(480,0)
-    #ax1.set_xlim(0,640)
-    cb = plt.colorbar()
-    #plt.xticks(rotation=45)
-    #plt.yticks(rotation=90)
+    if type(ax) == type(None): 
+        plt.figure(figsize=(wSize, hSize), dpi=300)
+        plt.title(figureTitle, fontsize=fontsize)
+        plt.scatter(x, y, c=c, cmap=cmap, s=3)
+        plt.plot([esquerda,esquerda] , [baixo,cima],"k")
+        plt.plot([esquerda,direita]  , [cima,cima],"k")
+        plt.plot([direita,direita]   , [cima,baixo],"k")
+        plt.plot([direita,esquerda]  , [baixo,baixo],"k")
+        cb = plt.colorbar()
 
-    if invertY == True:
-        plt.gca().invert_yaxis()
-    cb.set_label('Time (s)',fontsize=fontsize)
-    cb.ax.tick_params(labelsize=fontsize*0.8)
-    plt.xlabel("X (px)",fontsize=fontsize)
-    plt.ylabel("Y (px)",fontsize=fontsize)
-    plt.xticks(fontsize = fontsize*0.8)
-    plt.yticks(fontsize = fontsize*0.8)
-    if saveName != None:
-        plt.savefig(saveName+figformat)
-    plt.show()
+        if invertY == True:
+            plt.gca().invert_yaxis()
+        cb.set_label('Time (s)',fontsize=fontsize)
+        cb.ax.tick_params(labelsize=fontsize*0.8)
+        plt.xlabel("X (px)",fontsize=fontsize)
+        plt.ylabel("Y (px)",fontsize=fontsize)
+        plt.xticks(fontsize = fontsize*0.8)
+        plt.yticks(fontsize = fontsize*0.8)
+        plt.show()
 
+        if saveName != None:
+            plt.savefig(saveName+figformat)
 
-def Heatmap(data,bodyPart,cmapType='viridis',figureTitle=None,hSize=6,wSize =8,
-            bins=40,vmax=1000,fontsize=15,invertY=True,saveName=None,figformat=".eps"):
+    else:
+        plt.gca().set_aspect('equal', adjustable='box')
+        ax.scatter(x, y, c=c, cmap=cmap, s=3)
+        ax.plot([esquerda,esquerda] , [baixo,cima],"k")
+        ax.plot([esquerda,direita]  , [cima,cima],"k")
+        ax.plot([direita,direita]   , [cima,baixo],"k")
+        ax.plot([direita,esquerda]  , [baixo,baixo],"k")
+        ax.tick_params(axis='both', which='major', labelsize=fontsize)
+
+        if invertY == True:
+            ax.invert_yaxis()
+
+def Heatmap(data, bodyPart, **kwargs):
     """
     Plots the trajectory heatmap of the determined body part.
 
@@ -105,6 +160,14 @@ def Heatmap(data,bodyPart,cmapType='viridis',figureTitle=None,hSize=6,wSize =8,
         The input tracking data.
     bodyPart : str
         Body part you want to plot the heatmap.
+    start : int, optional
+        Moment of the video you want tracking to start, in seconds. If the variable 
+        is empty (None), the entire video will be processed.
+    end : int, optional
+        Moment of the video you want tracking to end, in seconds. If the variable is 
+        empty (None), the entire video will be processed.
+    fps : int
+        The recording frames per second.
     cmapType : str, optional
         matplotlib colormap.
     figureTitle : str, optional
@@ -113,6 +176,8 @@ def Heatmap(data,bodyPart,cmapType='viridis',figureTitle=None,hSize=6,wSize =8,
         Determine the figure height size (x).
     wSize : int, optional
         Determine the figure width size (y).
+    ax : fig axs, optional
+        Allows the creation of an out-of-function figure to use this plot.
     bins : int, optional
         Determine the heatmap resolution, the higher the value, the higher the 
         resolution.
@@ -145,31 +210,86 @@ def Heatmap(data,bodyPart,cmapType='viridis',figureTitle=None,hSize=6,wSize =8,
     import matplotlib.pyplot as plt
     import pandas as pd
     from matplotlib import cm
+    from mpl_toolkits.axes_grid1 import make_axes_locatable
+    
+    saveName= kwargs.get('saveName')
+    start= kwargs.get('start')
+    end= kwargs.get('end')
+    figureTitle = kwargs.get('figureTitle')
+    fps = kwargs.get('fps')
+    ax = kwargs.get('ax')
+    if type(fps) == type(None):
+      fps = 30
+    cmapType = kwargs.get('cmapType')
+    if type(cmapType) == type(None):
+      cmapType = 'viridis'
+    hSize = kwargs.get('hSize')
+    if type(hSize) == type(None):
+      hSize = 6
+    wSize = kwargs.get('wSize')
+    if type(wSize) == type(None):
+      wSize = 8
+    bins = kwargs.get('bins')
+    if type(bins) == type(None):
+      bins = 30
+    fontsize = kwargs.get('fontsize')
+    if type(fontsize) == type(None):
+      fontsize = 15
+    invertY = kwargs.get('invertY')
+    if type(invertY) == type(None):
+      invertY = True
+    figformat = kwargs.get('figformat')
+    if type(figformat) == type(None):
+      figformat = '.eps'
+    vmax = kwargs.get('vmax')
+    if type(vmax) == type(None):
+      vmax = 1000
+
 
     values = (data.iloc[2:,1:].values).astype(np.float)
     lista1 = (data.iloc[0][1:].values +" - " + data.iloc[1][1:].values).tolist()
 
-    x = values[:,lista1.index(bodyPart+" - x")]
-    y = values[:,lista1.index(bodyPart+" - y")]
+    if type(start) == type(None):
+        x = values[:,lista1.index(bodyPart+" - x")]
+        y = values[:,lista1.index(bodyPart+" - y")]
+    else:
+        init = int(start*fps)
+        finish = int(end*fps)
+        x = values[:,lista1.index(bodyPart+" - x")][init:finish]
+        y = values[:,lista1.index(bodyPart+" - y")][init:finish]
 
-    plt.rcParams["font.family"] = "Arial"
-    plt.figure(figsize=(wSize, hSize), dpi=80)
-    plt.hist2d(x,y, bins = bins, vmax = vmax,cmap=plt.get_cmap(cmapType))
+    if type(ax) == type(None):
+        plt.figure(figsize=(wSize, hSize), dpi=80)
+        plt.hist2d(x,y, bins = bins, vmax = vmax,cmap=plt.get_cmap(cmapType))
 
-    cb = plt.colorbar()
+        cb = plt.colorbar()
 
-    plt.title(figureTitle, fontsize=fontsize)
-    cb.ax.tick_params(labelsize=fontsize*0.8)
-    plt.xlabel("X (px)",fontsize=fontsize)
-    plt.ylabel("Y (px)",fontsize=fontsize)
-    plt.xticks(fontsize = fontsize*0.8)
-    plt.yticks(fontsize = fontsize*0.8)
-    if invertY == True:
-        plt.gca().invert_yaxis()
+        plt.title(figureTitle, fontsize=fontsize)
+        cb.ax.tick_params(labelsize=fontsize*0.8)
+        plt.xlabel("X (px)",fontsize=fontsize)
+        plt.ylabel("Y (px)",fontsize=fontsize)
+        plt.xticks(fontsize = fontsize*0.8)
+        plt.yticks(fontsize = fontsize*0.8)
+        if invertY == True:
+            plt.gca().invert_yaxis()
+        plt.show()
+
+        if saveName != None:
+            plt.savefig(saveName+figformat)
+    else:
+        ax.hist2d(x,y, bins = bins, vmax = vmax,cmap=plt.get_cmap(cmapType))
+        ax.tick_params(axis='both', which='major', labelsize=fontsize)
+
+        if invertY == True:
+            ax.invert_yaxis()
+
+        divider = make_axes_locatable(ax)
+        cax = divider.append_axes('right',size='5%', pad=0.05)
+
+        im = ax.imshow([x,y], cmap=plt.get_cmap(cmapType))
+        cb = fig.colorbar(im,cax=cax, orientation='vertical')
+        cb.ax.tick_params(labelsize=fontsize)
 
-    if saveName != None:
-        plt.savefig(saveName+figformat)
-    plt.show()
 
 def ScaleConverter(dado, pixel_max,pixel_min,max_real, min_real=0):
     """
@@ -251,7 +371,7 @@ def MotionMetrics (data,bodyPart,filter=0,fps=30,max_real=60,min_real=0):
     dataY = ScaleConverter(dataY,dataY.max(),dataY.min(), min_real,0)
 
     time = np.arange(0,((1/fps)*len(dataX)), (1/fps))
-    df = pd.DataFrame(time, columns = ["Time"])
+    df = pd.DataFrame(time/60, columns = ["Time"])
     dist = np.hypot(np.diff(dataX, prepend=dataX[0]), np.diff(dataY, prepend=dataY[0]))
     dist[dist>=filter] = 0
     dist[0] = "nan"
@@ -317,7 +437,7 @@ def FieldDetermination(Fields):
         
     return fields
 
-def Interaction(data,bodyPart,fields):
+def Interaction(data,bodyPart,fields,fps=30):
     """
     Performs the metrification of the interaction of the point of the determined 
     body part and the marked area.
@@ -329,7 +449,9 @@ def Interaction(data,bodyPart,fields):
     bodyPart : str
         Body part you want use as reference.
     fields : pandas DataFrame
-        The DataFrame with the coordinates of the created fields (output of FieldDetermination()).   
+        The DataFrame with the coordinates of the created fields (output of FieldDetermination()).
+    fps : int, optional
+        The recording frames per second.   
 
     Returns
     -------
@@ -370,7 +492,7 @@ def Interaction(data,bodyPart,fields):
     obj = 0
     start = 0
     end = 0
-    fps =30
+    fps =fps
 
     for i in range(len(interact)):
         if obj != interact[i]:
@@ -388,7 +510,7 @@ def Interaction(data,bodyPart,fields):
 
     return interactsDf
 
-def Reports(df_list,list_name,bodypart,fields=None,filter=0.3,fps = 30):
+def Reports(df_list,list_name,bodypart,fields=None,filter=0.3,fps=30):
     """
     Produces a report of all data passed along the way with movement and interaction metrics in a 
     given box space.
@@ -427,7 +549,7 @@ def Reports(df_list,list_name,bodypart,fields=None,filter=0.3,fps = 30):
     import pandas as pd
     import pyratlib as rat
 
-    relatorio = pd.DataFrame(columns=['file','time','dist', 'speed'])
+    relatorio = pd.DataFrame(columns=['file','video time (min)','dist (cm)', 'speed (cm/s)'])
   
     if type(fields) != type(None):  
         for i in range(len(fields)):
